/***************************************************************************
 *   Copyright (C) 2007 by Tobias Schwarz   *
 *   mail@tobias-schwarz.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/

#include <QtGui>
#include "configDlg.h"
#include "kembossWindowClass.h"

configDlg::configDlg(kemboss *parent, kembossSettings *fksettings) : QDialog(parent)
{
   setupUi(this);
   StyleComboBox->addItem("default");
   StyleComboBox->addItems(QStyleFactory::keys());
   mainKembossWindow = parent;
   ksettings = fksettings;
   fontHasChanged = false;
   connect(FontComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(fontChanged()));
   connect(FontSizeSpinBox, SIGNAL(valueChanged(int)), this, SLOT(fontChanged()));
   connect(WrapLinesCheckBox, SIGNAL(stateChanged(int)), this, SLOT(fontChanged()));
   embassyHasChanged = false;
   connect(PhylipnewCheckBox, SIGNAL(stateChanged(int)), this, SLOT(embassyChanged()));
   readSettings();

   KembossDataLineEdit->setText(ksettings->kembossData());
   KembossDefaultSettingsLineEdit->setText(ksettings->kembossDefaultSettings());
   EmbossBinLineEdit->setText(ksettings->embossBin());
   EmbossHelpLineEdit->setText(ksettings->embossHelp());
   EmbossDataLineEdit->setText(ksettings->embossData());
   OpenLastSettingsCheckBox->setChecked(ksettings->openLastSettings());
   CloseProgressWindowCheckBox->setChecked(ksettings->autoCloseProgressWindow());
   ShowOutputTabCheckBox->setChecked(ksettings->autoShowOutputTab());
   AutoDeleteExistingOutputFileCheckBox->setChecked(ksettings->autoDeleteExistingOutputFile());
   FontComboBox->setCurrentFont(ksettings->textEditFont());
   FontSizeSpinBox->setValue(ksettings->textEditFont().pointSize());
   WrapLinesCheckBox->setChecked(ksettings->wrapLines());
   getAvailableDatabases();
}

void configDlg::accept()
{
   ksettings->setkembossData(KembossDataLineEdit->text());
   ksettings->setkembossDefaultSettings(KembossDefaultSettingsLineEdit->text());
   ksettings->setembossBin(EmbossBinLineEdit->text());
   ksettings->setembossHelp(EmbossHelpLineEdit->text()); 
   ksettings->setembossData(EmbossDataLineEdit->text());
   ksettings->setopenLastSettings(OpenLastSettingsCheckBox->checkState());
   ksettings->setautoCloseProgressWindow(CloseProgressWindowCheckBox->checkState());
   ksettings->setautoShowOutputTab(ShowOutputTabCheckBox->checkState());
   ksettings->setautoDeleteExistingOutputFile(AutoDeleteExistingOutputFileCheckBox->checkState());
   ksettings->settextEditFont(FontComboBox->currentFont().family(), FontSizeSpinBox->value());
   ksettings->setwrapLines(WrapLinesCheckBox->checkState());
   ksettings->writeSettings();
   writeSettings();
   if (fontHasChanged)
      foreach (kembossWindow *window, mainKembossWindow->programWindows)
         qobject_cast<kprogramWindow *>(window)->setTextEditFont();
   if (embassyHasChanged)
       mainKembossWindow->loadLists(PhylipnewCheckBox->checkState());
   QDialog::accept();
}

void configDlg::readSettings()
{
   QSettings settings("kemboss.org", "configDlg");
   QPoint pos = settings.value("conf_pos").toPoint();
   QSize size = settings.value("conf_size").toSize();
   if (!size.isEmpty()) resize(size);
   if (!pos.isNull()) move(pos);
   DatabaseTypeComboBox->setCurrentIndex(settings.value("dbindex_prog").toInt());
   IDCheckBox->setChecked(settings.value("id", false).toBool());
   AccNoCheckBox->setChecked(settings.value("acc_no", false).toBool());
   SVAndGICheckBox->setChecked(settings.value("sv_and_gi", false).toBool());
   DescriptionCheckBox->setChecked(settings.value("description", false).toBool());
   KeywordsCheckBox->setChecked(settings.value("keywords", false).toBool());
   TaxonomyCheckBox->setChecked(settings.value("taxonomy", false).toBool());
   DatabaseNameLineEdit->setText(settings.value("db_name").toString());
   ResourceNameLineEdit->setText(settings.value("resource_name").toString());
   FormatComboBox->setCurrentIndex(settings.value("id_format").toInt());
   ReleaseLineEdit->setText(settings.value("release", "0.0").toString());
   IndexDateLineEdit->setText(QDate::currentDate().toString("dd/MM/yy"));
   BlastIndexVersionComboBox->setCurrentIndex(settings.value("blast_index_version").toInt());
   UseFASTASourceFileCheckBox->setChecked(settings.value("use_fasta_source", false).toBool());
   UseSystemSortUtilityCheckBox->setChecked(settings.value("use_system_sort", true).toBool());
   CleanUpTemporaryFilesCheckBox->setChecked(settings.value("clean_up", true).toBool());
   MaximumIndexLengthSpinBox->setValue(settings.value("max_index_length").toInt());
   SortOptionsLineEdit->setText(settings.value("sort_options", "-T . -k 1,1").toString());
   DatabaseDirectoryLineEdit->setText(settings.value("db_dir").toString());
   DatabaseFilenameLineEdit->setText(settings.value("db_filename", "*.dat").toString());
   WildcardFilenamesToExcludeLineEdit->setText(settings.value("wildcard_filenames").toString());
   IndexDirectoryLineEdit->setText(settings.value("index_dir", ".").toString());
   LogOutputFileLineEdit->setText(settings.value("log_output_file", "dbi.log").toString());

   QSettings kembossSettings("kemboss.org", "kemboss");
   PhylipnewCheckBox->setChecked(kembossSettings.value("phylipnew_installed", false).toBool());
   QString currentStyle = kembossSettings.value("style", "default").toString();
   int i = 0;
   while (i < StyleComboBox->count()) {
      if (StyleComboBox->itemText(i) == currentStyle) {
         StyleComboBox->setCurrentIndex(i);
         break;
      }
      i++;
   }
}

void configDlg::writeSettings()
{
   QSettings settings("kemboss.org", "configDlg");
   settings.setValue("conf_pos", pos());
   settings.setValue("conf_size", size());
   settings.setValue("dbindex_prog", DatabaseTypeComboBox->currentIndex());
   settings.setValue("id", IDCheckBox->checkState());
   settings.setValue("acc_no", AccNoCheckBox->checkState());
   settings.setValue("sv_and_gi", SVAndGICheckBox->checkState());
   settings.setValue("description", DescriptionCheckBox->checkState());
   settings.setValue("keywords", KeywordsCheckBox->checkState());
   settings.setValue("taxonomy", TaxonomyCheckBox->checkState());
   settings.setValue("db_name", DatabaseNameLineEdit->text());
   settings.setValue("resource_name", ResourceNameLineEdit->text());
   settings.setValue("id_format", FormatComboBox->currentIndex());
   settings.setValue("release", ReleaseLineEdit->text());
   settings.setValue("blast_index_version", BlastIndexVersionComboBox->currentIndex());
   settings.setValue("use_fasta_source", UseFASTASourceFileCheckBox->checkState());
   settings.setValue("use_system_sort", UseSystemSortUtilityCheckBox->checkState());
   settings.setValue("clean_up", CleanUpTemporaryFilesCheckBox->checkState());
   settings.setValue("max_index_length", MaximumIndexLengthSpinBox->value());
   settings.setValue("sort_options", SortOptionsLineEdit->text());
   settings.setValue("db_dir", DatabaseDirectoryLineEdit->text());
   settings.setValue("db_filename", DatabaseFilenameLineEdit->text());
   settings.setValue("wildcard_filenames", WildcardFilenamesToExcludeLineEdit->text());
   settings.setValue("index_dir", IndexDirectoryLineEdit->text());
   settings.setValue("log_output_file", LogOutputFileLineEdit->text());

   QSettings kembossSettings("kemboss.org", "kemboss");
   kembossSettings.setValue("phylipnew_installed", PhylipnewCheckBox->checkState());
}

void configDlg::on_StyleComboBox_activated(const QString &text)
{
   if (text != "default") {
      QApplication::setStyle(QStyleFactory::create(text));
      QApplication::setPalette(QApplication::style()->standardPalette());
   } else QMessageBox::information(this, "kemboss", "The default style will be used, when kemboss is restarted.");
      QSettings settings("kemboss.org", "kemboss");
      settings.setValue("style", text);
}

void configDlg::setExistingDir(const QString &dlgMsg, QLineEdit *lineEdit)
{
   QString dirName = QDir::toNativeSeparators(QFileDialog::getExistingDirectory(this, dlgMsg, ksettings->currentDir(), QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks));
   if (!dirName.isEmpty()) {
      lineEdit->setText(dirName);
      ksettings->setcurrentDir(dirName);
   }
}

void configDlg::setOpenFile(const QString &dlgMsg, QLineEdit *lineEdit)
{
   QString fileName = QFileDialog::getOpenFileName(this, dlgMsg, ksettings->currentDir());
   if (!fileName.isEmpty()) {
      lineEdit->setText(fileName);
      ksettings->setcurrentDir(QFileInfo(fileName).absolutePath());
   }
}

void configDlg::setSaveFile(const QString &dlgMsg, QLineEdit *lineEdit)
{
   QString fileName = QFileDialog::getSaveFileName(this, dlgMsg, ksettings->currentDir());
   if (!fileName.isEmpty()) {
      lineEdit->setText(fileName);
      ksettings->setcurrentDir(QFileInfo(fileName).absolutePath());
   }
}

void configDlg::extractDbChanged()
{
   REBASEGroupBox->setChecked(false);
   CUTGGroupBox->setChecked(false);
   AAINDEXGroupBox->setChecked(false);
   JASPARGroupBox->setChecked(false);
   PRINTSGroupBox->setChecked(false);
   PROSITEGroupBox->setChecked(false);
   TRANSFACGroupBox->setChecked(false);
   if (database == "rebase")
      REBASEGroupBox->setChecked(true);
   else if (database == "cutg")
      CUTGGroupBox->setChecked(true);
   else if (database == "aaindex")
      AAINDEXGroupBox->setChecked(true);
   else if (database == "pros")
      PROSITEGroupBox->setChecked(true);
   else if (database == "jasp")
      JASPARGroupBox->setChecked(true);
   else if (database == "tf")
      TRANSFACGroupBox->setChecked(true);
   else if (database == "prints")
      PRINTSGroupBox->setChecked(true);
}

void configDlg::on_OpenHomepagePushButton_clicked()
{
   if (database == "rebase")
      QDesktopServices::openUrl (QUrl("http://rebase.neb.com/"));
   else if (database == "cutg")
      QDesktopServices::openUrl (QUrl("ftp://ftp.ebi.ac.uk/pub/databases/cutg/"));
   else if (database == "aaindex")
      QDesktopServices::openUrl (QUrl("ftp://ftp.genome.ad.jp/pub/db/genomenet/aaindex/"));
   else if (database == "pros")
      QDesktopServices::openUrl (QUrl(""));
   else if (database == "jasp")
      QDesktopServices::openUrl (QUrl("http://jaspar.genereg.net/"));
   else if (database == "tf")
      QDesktopServices::openUrl (QUrl("http://www.gene-regulation.com/pub/databases.html"));
   else if (database == "prints")
      QDesktopServices::openUrl (QUrl("http://www.bioinf.man.ac.uk/dbbrowser/PRINTS/"));
}

void configDlg::on_ExtractDatabasePushButton_clicked()
{
   QStringList programConfiguration;
   if (database == "rebase") {
      if (WithrefmFileLineEdit->text().isEmpty() || !QFile::exists(WithrefmFileLineEdit->text())) {
         QMessageBox::information(this, "kemboss", tr("Please select correct withrefm file."));
         return;
      }
      if (ProtoFileLineEdit->text().isEmpty() || !QFile::exists(ProtoFileLineEdit->text())) {
         QMessageBox::information(this, "kemboss", tr("Please select correct proto file."));
         return;
      }
      programConfiguration.append(WithrefmFileLineEdit->text());
      programConfiguration.append(ProtoFileLineEdit->text());
      if (!CalculateEquFileCheckBox->isChecked()) programConfiguration.append("-noequivalences");
   } else if (database == "cutg") {
      QDir cutgDir(CUTGDirectoryLineEdit->text());
      if (CUTGDirectoryLineEdit->text().isEmpty() || !cutgDir.exists()) {
         QMessageBox::information(this, "kemboss", tr("Please select correct CUTG data directory."));
         return;
      }
      if (GeneralLogOutputFileLineEdit->text().isEmpty()) {
         QMessageBox::information(this, "kemboss", tr("Please select correct CUTG log output file."));
         return;
      }
      programConfiguration.append(CUTGDirectoryLineEdit->text());
      programConfiguration.append(GeneralLogOutputFileLineEdit->text());
      if (!CUTGReleaseLineEdit->text().isEmpty()) programConfiguration.append("-release "+CUTGReleaseLineEdit->text());
      if (!TypeOfCodonFileLineEdit->text().isEmpty()) programConfiguration.append("-wildspec "+TypeOfCodonFileLineEdit->text());
      if (!SpeciesWildcardLineEdit->text().isEmpty()) programConfiguration.append("-species "+SpeciesWildcardLineEdit->text());
      if (!SingleOutputFilenameLineEdit->text().isEmpty()) programConfiguration.append("-filename "+SingleOutputFilenameLineEdit->text());
      if (IncludeAllRecordsCheckBox->isChecked()) programConfiguration.append("-allrecords");
   } else if (database == "aaindex") {
      if (AAINDEXFileLineEdit->text().isEmpty() || !QFile::exists(AAINDEXFileLineEdit->text())) {
         QMessageBox::information(this, "kemboss", tr("Please select correct AAINDEX file."));
         return;
      }
      programConfiguration.append(AAINDEXFileLineEdit->text());
   } else if (database == "pros") {
      QDir prosDir(PROSITEDirectoryLineEdit->text());
      if (PROSITEDirectoryLineEdit->text().isEmpty() || !prosDir.exists()) {
         QMessageBox::information(this, "kemboss", tr("Please select the correct PROSITE database directory."));
         return;
      }
      programConfiguration.append(PROSITEDirectoryLineEdit->text());
   } else if (database == "jasp") {
      QDir jaspDir(JASPARDirectoryLineEdit->text());
      if (JASPARDirectoryLineEdit->text().isEmpty() || !jaspDir.exists()) {
         QMessageBox::information(this, "kemboss", tr("Please select the correct JASPAR data directory."));
         return;
      }
      programConfiguration.append(JASPARDirectoryLineEdit->text());
   } else if (database == "tf") {
      if (TRANSFACFileLineEdit->text().isEmpty() || !QFile::exists(TRANSFACFileLineEdit->text())) {
         QMessageBox::information(this, "kemboss", tr("Please select correct TRANSFAC 'site.dat' file."));
         return;
      }
      programConfiguration.append(TRANSFACFileLineEdit->text());
   } else if (database == "prints") {
      if (PRINTSFileLineEdit->text().isEmpty() || !QFile::exists(PRINTSFileLineEdit->text())) {
         QMessageBox::information(this, "kemboss", tr("Please select correct PRINTS database file."));
         return;
      }
      programConfiguration.append(PRINTSFileLineEdit->text());
   }
   extractDatabase(database+"extract", programConfiguration);
}

void configDlg::extractDatabase(const QString &program, const QStringList &config)
{
   QProcess dbProcess;
   dbProcess.setProcessChannelMode(QProcess::MergedChannels);
   dbProcess.start(ksettings->embossBin()+program
   #ifdef Q_WS_WIN
   +".exe"
   #endif
   , config);
   if (dbProcess.waitForFinished()) { // waits 30 sec. for program to finish
      QByteArray programOutput = dbProcess.readAll();
      QMessageBox::information(this, "kemboss", QString(programOutput)+"\nFinished");
   } else QMessageBox::warning(this, "kemboss", tr("Extraction of database failed!"));
}

void configDlg::getAvailableDatabases() {
   AvailableDatabasesTextEdit->document()->clear();
   QProcess showdb;
   showdb.start(ksettings->embossBin()+"showdb -full");
   if (!showdb.waitForFinished()) return; // waits 30 sec. for program showdb to finish
   QByteArray programOutput = showdb.readAll();
   AvailableDatabasesTextEdit->setText(QString(programOutput));
}

void configDlg::on_AddDBPushButton_clicked()
{
   if (NameLineEdit->text().isEmpty()) {
      QMessageBox::warning(this, "kemboss", tr("Please, enter a name for the database."));
      return;
   }
   QString fileName;
   if (WriteToComboBox->currentIndex() == 1) { //Write to emboss.default
      if (QFile::exists(ksettings->embossBin()+"emboss.default")) fileName = ksettings->embossBin()+"emboss.default"; //Default on MS Windows
      else if (QFile::exists(ksettings->embossData()+"../emboss.default")) fileName = ksettings->embossData()+"../emboss.default";  //Default on UNIX
      if (fileName.isEmpty()) {
         QMessageBox::warning(this, "kemboss", tr("emboss.default file not found!\nCreating an emboss.default file in the parent directory of the EMBOSS data directory."));
         fileName = ksettings->embossData()+"../emboss.default";
      }
   } else { //Write to .embossrc in users home directory
      fileName = QDir::homePath()+QDir::separator()+".embossrc";
   }
   QFile file(fileName);
   if (!file.open(QFile::Append | QFile::Text)) {
      QMessageBox::warning(this, objectName(),
                   tr("Cannot write file %1:\n%2.")
                   .arg(fileName)
                   .arg(file.errorString()));
      return;
   }
   QTextStream out(&file);
   out << "\n";
   out << "#Database definition created by kemboss\n";
   out << "DB "+NameLineEdit->text()+" [\n";
   out << "   method: \""+MethodComboBox->currentText()+"\"\n";
   out << "   type: \""+TypeComboBox->currentText()+"\"\n";
   out << "   format: \""+FormatComboBox_2->currentText()+"\"\n";
   if (!FieldsLineEdit->text().isEmpty()) out << "   fields: \""+FieldsLineEdit->text()+"\"\n";
   if (!ReleaseLineEdit_2->text().isEmpty()) out << "   release: \""+ReleaseLineEdit_2->text()+"\"\n";
   if (!DBAliasLineEdit->text().isEmpty()) out << "   dbalias: \""+DBAliasLineEdit->text()+"\"\n";
   if (!URLLineEdit->text().isEmpty()) out << "   url: \""+URLLineEdit->text()+"\"\n";
   if (!AppLineEdit->text().isEmpty()) out << "   app: \""+AppLineEdit->text()+"\"\n";
   if (!DirectoryLineEdit->text().isEmpty()) out << "   dir: \""+DirectoryLineEdit->text()+"\"\n";
   if (!FileLineEdit->text().isEmpty()) out << "   file: \""+FileLineEdit->text()+"\"\n";
   if (!ExludeLineEdit->text().isEmpty()) out << "   exclude: \""+ExludeLineEdit->text()+"\"\n";
   if (!IndexDirLineEdit->text().isEmpty()) out << "   indexdir: \""+IndexDirLineEdit->text()+"\"\n";
   if (!CommentLineEdit->text().isEmpty()) out << "   comment: \""+CommentLineEdit->text()+"\"\n";
   out << "]\n";
   getAvailableDatabases();
}

void configDlg::on_DatabaseTypeComboBox_currentIndexChanged(const QString &text)
{
   bool tmpBool;
   if (text == "dbiblast") tmpBool = true; else tmpBool = false;
      BlastIndexVersionLabel->setEnabled(tmpBool);
      BlastIndexVersionComboBox->setEnabled(tmpBool);
      UseFASTASourceFileCheckBox->setEnabled(tmpBool);
   if (text.startsWith("dbx")) tmpBool = true; else tmpBool = false;
      ResourceNameLabel->setEnabled(tmpBool);
      ResourceNameLineEdit->setEnabled(tmpBool);
      SystemGroupBox->setEnabled(!tmpBool);
      LogOutputFileLineEdit->setEnabled(!tmpBool);
      LogOutputFileLabel->setEnabled(!tmpBool);
      LogOutputFilePushButton->setEnabled(!tmpBool);

   FormatComboBox->clear();
   QStringList idItems;
   if (text == "dbiblast") {
      FormatLabel->setText("Sequence type:");
      idItems << "N (nucleic)" << "? (unknown)" << "P (protein)";
   } else if (text == "dbifasta" || text == "dbxfasta") {
      FormatLabel->setText("ID format:");
      idItems << "simple (>ID)" << "idacc (>ID ACC) or (>ID (ACC))" << "gcgid (>db:ID)" << "gcgidacc (>db:ID ACC)" << "dbid (>db ID)" << "ncbi (| formats)";
   } else if (text == "dbiflat" || text == "dbxflat") {
      FormatLabel->setText("Entry format:");
      idItems << "EMBL (EMBL)" << "SWISS (Swiss-Prot, SpTrEMBL, TrEMBLnew)" << "GB (Genbank, DDBJ)" << "REFSEQ (Refseq)";
   } else if (text == "dbigcg" || text == "dbxgcg") {
      FormatLabel->setText("Entry format:");
      idItems << "EMBL (EMBL)" << "SWISS (Swiss-Prot, SpTrEMBL, TrEMBLnew)" << "GENBANK (Genbank, DDBJ)" << "PIR (NBRF)";
   }
   if (text == "dbxfasta" || text == "dbxflat" || text == "dbxgcg") IDCheckBox->setEnabled(true); else IDCheckBox->setEnabled(false);
   if (text == "dbiflat" || text == "dbigcg" || text == "dbxflat" || text == "dbxgcg") {
      KeywordsCheckBox->setEnabled(true);
      TaxonomyCheckBox->setEnabled(true);
   } else {
      KeywordsCheckBox->setEnabled(false);
      TaxonomyCheckBox->setEnabled(false);
   }
   FormatComboBox->addItems(idItems);
   FormatComboBox->setCurrentIndex(1);
}

void configDlg::on_IndexDatabasePushButton_clicked()
{
   QStringList programConfiguration;

   // Configuration parameters for all database index programs
   if (!DatabaseNameLineEdit->text().isEmpty()) programConfiguration.append("-dbname="+DatabaseNameLineEdit->text());
   if (!DatabaseDirectoryLineEdit->text().isEmpty()) programConfiguration.append("-directory="+DatabaseDirectoryLineEdit->text());
   if (!DatabaseFilenameLineEdit->text().isEmpty()) programConfiguration.append("-filenames="+DatabaseFilenameLineEdit->text());
   if (!ReleaseLineEdit->text().isEmpty()) programConfiguration.append("-release="+ReleaseLineEdit->text());
   if (!IndexDateLineEdit->text().isEmpty()) programConfiguration.append("-date="+IndexDateLineEdit->text());
   if (!WildcardFilenamesToExcludeLineEdit->text().isEmpty()) programConfiguration.append("-exclude="+WildcardFilenamesToExcludeLineEdit->text());
   if (!IndexDirectoryLineEdit->text().isEmpty()) programConfiguration.append("-indexoutdir="+IndexDirectoryLineEdit->text());

   QString fieldsString = "";
   if (IDCheckBox->isEnabled() && IDCheckBox->isChecked()) fieldsString += "id,";
   if (AccNoCheckBox->isChecked()) fieldsString += "acc,";
   if (SVAndGICheckBox->isChecked()) fieldsString += "sv,";
   if (DescriptionCheckBox->isChecked()) fieldsString += "des,";
   if (KeywordsCheckBox->isEnabled() && KeywordsCheckBox->isChecked()) fieldsString += "key,";
   if (TaxonomyCheckBox->isEnabled() && TaxonomyCheckBox->isChecked()) fieldsString += "org,";
   if (!fieldsString.isEmpty()) {
      fieldsString.chop(1); // remove the last comma
      programConfiguration.append("-fields="+fieldsString);
   }
   // Configuration specific for the dbi... but not the dbx... programs
   if (DatabaseTypeComboBox->currentText().startsWith("dbi")) {
      if (!LogOutputFileLineEdit->text().isEmpty()) programConfiguration.append("-outfile="+LogOutputFileLineEdit->text());
      if (MaximumIndexLengthSpinBox->value() != 0) programConfiguration.append("-maxindex="+MaximumIndexLengthSpinBox->text());
      if (!SortOptionsLineEdit->text().isEmpty() && SortOptionsLineEdit->text() != "-T . -k 1,1") programConfiguration.append("-sortoptions=\""+SortOptionsLineEdit->text()+"\"");
      if (!UseSystemSortUtilityCheckBox->isChecked()) programConfiguration.append("-nosystemsort");
      if (!CleanUpTemporaryFilesCheckBox->isChecked()) programConfiguration.append("-nocleanup");
   } else {

   // Configuration specific for the dbx... but not the dbi... programs
      if (!ResourceNameLineEdit->text().isEmpty()) programConfiguration.append("-dbresource="+ResourceNameLineEdit->text());
   }

   // Program specific configuration
   if (DatabaseTypeComboBox->currentText() == "dbiblast") {
      programConfiguration.append("-seqtype="+FormatComboBox->currentText().section(" ", 0, 0));
      programConfiguration.append("-blastversion="+BlastIndexVersionComboBox->currentText().section(" ", 0, 0));
      if (UseFASTASourceFileCheckBox->isChecked()) programConfiguration.append("-sourcefile");
   } else programConfiguration.append("-idformat="+FormatComboBox->currentText().section(" ", 0, 0));

   QProcess indexDb;
   indexDb.setProcessChannelMode(QProcess::MergedChannels);
   indexDb.start(ksettings->embossBin()+DatabaseTypeComboBox->currentText()
   #ifdef Q_WS_WIN
   +".exe"
   #endif
   , programConfiguration);
   if (indexDb.waitForFinished(-1)) {
      getAvailableDatabases();
   }
   QByteArray programOutput = indexDb.readAll();
   QMessageBox::information(this, "kemboss", DatabaseTypeComboBox->currentText()+" "+programConfiguration.join(" ")+" "+QString(programOutput)+"Finished");
}
