/***************************************************************************
 *   Copyright (C) 2007 by Tobias Schwarz   *
 *   mail@tobias-schwarz.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/

#ifndef CONFIGDLG_H
#define CONFIGDLG_H

#include "settings.h"
#include "kemboss.h"
#include "ui_configDlg.h"

class configDlg : public QDialog, private Ui::ConfigDialog
{
    Q_OBJECT

public:
    configDlg(kemboss *parent = 0, kembossSettings *fksettings = 0);

private slots:
    void accept();
    void fontChanged() { fontHasChanged = true; };
    void embassyChanged() { embassyHasChanged = true; };
    void on_StyleComboBox_activated(const QString &text);
    void on_KembossDataPushButton_clicked() { setExistingDir(tr("Select kemboss data directory"), KembossDataLineEdit); }
    void on_KembossDefaultSettingsPushButton_clicked() { setExistingDir(tr("Select kemboss default settings directory"), KembossDefaultSettingsLineEdit); }
    void on_EmbossBinPushButton_clicked() { setExistingDir(tr("Select EMBOSS binaries directory"), EmbossBinLineEdit); }
    void on_EmbossHelpPushButton_clicked() { setExistingDir(tr("Select EMBOSS helpfiles directory"), EmbossHelpLineEdit); }
    void on_EmbossDataPushButton_clicked() { setExistingDir(tr("Select EMBOSS data directory"), EmbossDataLineEdit); }
    void on_REBASEGroupBox_clicked() { database = "rebase"; extractDbChanged(); }
    void on_CUTGGroupBox_clicked() { database = "cutg"; extractDbChanged(); }
    void on_AAINDEXGroupBox_clicked() { database = "aaindex"; extractDbChanged(); }
    void on_JASPARGroupBox_clicked() { database = "jasp"; extractDbChanged(); }
    void on_PRINTSGroupBox_clicked() { database = "prints"; extractDbChanged(); }
    void on_PROSITEGroupBox_clicked() { database = "pros"; extractDbChanged(); }
    void on_TRANSFACGroupBox_clicked() { database = "tf"; extractDbChanged(); }
    void on_DirectoryPushButton_clicked() { setExistingDir(tr("Select sequence database directory"), DirectoryLineEdit); }
    void on_IndexDirPushButton_clicked() { setExistingDir(tr("Select database index directory"), IndexDirLineEdit); }
    void on_AddDBPushButton_clicked();
    void on_WithrefmFilePushButton_clicked() { setOpenFile(tr("Select REBASE withrefm file"), WithrefmFileLineEdit); }
    void on_ProtoFilePushButton_clicked() { setOpenFile(tr("Select REBASE proto file"), ProtoFileLineEdit); }
    void on_CUTGDirectoryPushButton_clicked() { setExistingDir(tr("Select CUTG data directory"), CUTGDirectoryLineEdit); }
    void on_GeneralLogOutputFilePushButton_clicked() { setSaveFile(tr("Select general log output file"), AAINDEXFileLineEdit); }
    void on_AAINDEXFilePushButton_clicked() { setOpenFile(tr("Select AAINDEX database file"), TRANSFACFileLineEdit); }
    void on_PROSITEDirectoryPushButton_clicked() { setExistingDir(tr("Select PROSITE data directory"), PROSITEDirectoryLineEdit); }
    void on_JASPARDirectoryPushButton_clicked() { setExistingDir(tr("Select JASPAR data directory"), JASPARDirectoryLineEdit); }
    void on_TRANSFACFilePushButton_clicked() { setOpenFile(tr("Select 'site.dat' TRANSFAC file"), AAINDEXFileLineEdit); }
    void on_PRINTSFilePushButton_clicked() { setOpenFile(tr("Select PRINTS database file"), PRINTSFileLineEdit); }
    void on_OpenHomepagePushButton_clicked();
    void on_ExtractDatabasePushButton_clicked();
    void on_DatabaseDirectoryPushButton_clicked() { setExistingDir(tr("Select database directory"), DatabaseDirectoryLineEdit); }
    void on_LogOutputFilePushButton_clicked() { setSaveFile(tr("Select general log output file"), LogOutputFileLineEdit); }
    void on_IndexDirectoryPushButton_clicked() { setExistingDir(tr("Select index directory"), IndexDirectoryLineEdit); }
    void on_IndexDatabasePushButton_clicked();
    void on_DatabaseTypeComboBox_currentIndexChanged(const QString &text);

private:
    QString database;
    void readSettings();
    void writeSettings();
    void getAvailableDatabases();
    void extractDbChanged();
    void extractDatabase(const QString &program, const QStringList &config);
    void setExistingDir(const QString &dlgMsg, QLineEdit *lineEdit);
    void setOpenFile(const QString &dlgMsg, QLineEdit *lineEdit);
    void setSaveFile(const QString &dlgMsg, QLineEdit *lineEdit);
    bool fontHasChanged;
    bool embassyHasChanged;
    kemboss *mainKembossWindow;
    kembossSettings *ksettings;
};

#endif
