/***************************************************************************
 *   Copyright (C) 2007-2010 by Tobias Schwarz                             *
 *   mail@tobias-schwarz.net                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/

#include <QtGui>
#include "kemboss.h"
#include "programWindows.h"
#include "phylipProgramWindows.h"
#include "main.h"
#include "configDlg.h"

kemboss::kemboss(kembossWindow *parent) : kembossWindow(parent)
{
   setupUi(this);
   kembossParent = this;
   createActions();
   connect(actionOpen, SIGNAL(triggered()), this, SLOT(open()));
   connect(actionSave, SIGNAL(triggered()), this, SLOT(save()));
   connect(actionSaveAs, SIGNAL(triggered()), this, SLOT(saveAs()));
   connect(actionQuit, SIGNAL(triggered()), this, SLOT(close()));
   menu_Help->addAction(reportBugAct);
   menu_Help->addAction(aboutAct);
   recentSettingsMenu = menu_Recent_Settings;
   createRecentFiles();
   readSettings();
   if (!ksettings.checkSettings()) {
      QMessageBox::warning(this, "kemboss", tr("One or more EMBOSS directories are not specified. Please set the correct paths."));
      on_PreferencesPushButton_clicked();
   }
   QSettings settings("kemboss.org", "kemboss");
   loadLists(settings.value("phylipnew_installed", false).toBool());
   ProgramsTabWidget->setCurrentIndex(settings.value("view",1).toInt());
   ProgramsTreeWidget->setColumnWidth(0,settings.value("tree_widget1_width_col1",100).toInt());
   ProgramsTreeWidget->setColumnWidth(1,settings.value("tree_widget1_width_col2",200).toInt());
   ProgramsTreeWidget_2->setColumnWidth(0,settings.value("tree_widget2_width_col1",100).toInt());
   ProgramsTreeWidget_2->setColumnWidth(1,settings.value("tree_widget2_width_col2",200).toInt());
   ProgramsTreeWidget_3->setColumnWidth(0,settings.value("tree_widget3_width_col1",100).toInt());
   ProgramsTreeWidget_3->setColumnWidth(1,settings.value("tree_widget3_width_col2",200).toInt());
}

void kemboss::loadLists(const bool phylipnew)
{
   ProgramsTreeWidget->clear();
   createProgramList("programs_list");
   if (phylipnew)
      createProgramList("phylipnew_programs_list");
   ProgramsTreeWidget->sortItems(0,Qt::AscendingOrder);

   ProgramsTreeWidget_2->clear();
   if (phylipnew)
      createProgramTree("programs_tree_phylipnew");
   else createProgramTree("programs_tree");
}

void kemboss::createProgramList(const QString &programsFile)
// Parse output created by "wossname '' -alphabetic -noembassy -gui -outfile='programs_list'"
{
   QFile file(ksettings.kembossData() + programsFile);
   if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
      QMessageBox::warning(this, "kemboss", tr("Couldn't open file ")+programsFile+tr("! Please check the settings for the kemboss data files."));
      return;
   }
   QTextStream in(&file);
   while (!in.atEnd()) {
      QString line = in.readLine();
      ProgramsTreeWidget->addTopLevelItem(new QTreeWidgetItem(ProgramsTreeWidget, QStringList() << line.section(" ",0,0) << line.section(" ",1).trimmed()));
   }
}

void kemboss::createProgramTree(const QString &programsFile)
// Parse output created by "wossname '' -noembassy -colon -gui -outfile='programs_tree'"
{
   QFile file(ksettings.kembossData() + programsFile);
   if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
      QMessageBox::warning(this, "kemboss", tr("Couldn't open file ")+programsFile+tr("! Please check the settings for the kemboss data files."));
      return;
   }
   QTextStream in(&file);
   bool nextLineContainsGroup = true; // The first line contains a group description
   QString mainGroupName = "";
   QString mainGroup;
   QString subGroup;
   QTreeWidgetItem *mainGroupItem = NULL;
   QTreeWidgetItem *subGroupItem = NULL;
   while (!in.atEnd()) {
      QString line = in.readLine();
      if (line.isEmpty()) {
         nextLineContainsGroup = true; // When a line is empty, the next line contains a group description
      } else {
         if (nextLineContainsGroup) {  // Line contains a group description
            if (line.contains(":")) {  // A SubGroup belongs to a MainGroup
               mainGroup = line.section(":",0,0);
               subGroup = line.section(":",1,1);
               if (mainGroup != mainGroupName) { // MainGroup does not yet exist
                  mainGroupItem = new QTreeWidgetItem(ProgramsTreeWidget_2, QStringList() << mainGroup << "");
                  mainGroupName = mainGroup;
               }
               subGroupItem = new QTreeWidgetItem(mainGroupItem, QStringList() << subGroup << "");
            } else
               subGroupItem = new QTreeWidgetItem(ProgramsTreeWidget_2, QStringList() << line << "");
         } else  // Line contains program
            new QTreeWidgetItem(subGroupItem, QStringList() << line.section(" ",0,0) << line.section(" ",1).trimmed());
         nextLineContainsGroup = false;
      }
   }
}

QString kemboss::selectedProgram()
{
   if (ProgramsTabWidget->currentIndex() == 0) {
      if (ProgramsTreeWidget->currentItem() != NULL) return ProgramsTreeWidget->currentItem()->text(0);
   } else if (ProgramsTabWidget->currentIndex() == 1) {
      if (ProgramsTreeWidget_2->currentItem() != NULL) return ProgramsTreeWidget_2->currentItem()->text(0);
   } else if (ProgramsTabWidget->currentIndex() == 2) {
      if (ProgramsTreeWidget_3->currentItem() != NULL) return ProgramsTreeWidget_3->currentItem()->text(0);
   }
   return "";
}

bool kemboss::loadFile(const QString &fileName)
{
   statusBar()->showMessage(tr("Opening settings"));

   QFile file(fileName);
   if (!file.open(QFile::ReadOnly | QFile::Text)) {
      QMessageBox::warning(this, objectName(),
                    tr("Cannot read file %1:\n%2.")
                    .arg(fileName)
                    .arg(file.errorString()));
      return false;
   }

   // Load program windows, their current filenames and positions on the desktop
   QSettings settings(fileName, QSettings::IniFormat);
   int size = settings.beginReadArray("open_program_windows");
   for (int i = 0; i < size; ++i) {
      settings.setArrayIndex(i);
      kprogramWindow *newProgramWindow = createProgramWindow(settings.value("program").toString());
      QString fileName = settings.value("current_settings").toString();
      if (QFile::exists(fileName) && newProgramWindow->loadFile(fileName))
         newProgramWindow->setCurrentFile(fileName);
      QPoint pos = settings.value("pos").toPoint();
      QSize size = settings.value("size").toSize();
      if (!size.isEmpty()) newProgramWindow->resize(size);
      if (!pos.isNull()) newProgramWindow->move(pos);
   }
   settings.endArray();
   statusBar()->clearMessage();
   return true;
}

bool kemboss::saveFile(const QString &fileName)
{
   statusBar()->showMessage(tr("Saving settings"));
   QFile file(fileName);
   if (!file.open(QFile::WriteOnly | QFile::Text)) {
      QMessageBox::warning(this, "kemboss",
                   tr("Cannot write file %1:\n%2.")
                   .arg(fileName)
                   .arg(file.errorString()));
      return false;
   }

   // Save open program windows, their current filenames and positions on the desktop
   QSettings settings(fileName, QSettings::IniFormat);
   settings.setValue("kemboss_version",KEMBOSS_VERSION);

   settings.beginWriteArray("open_program_windows");
   for (int i = 0; i < programWindows.size(); ++i) {
      settings.setArrayIndex(i);
      settings.setValue("program", qobject_cast<kprogramWindow *>(programWindows.at(i))->embossApp());
      settings.setValue("current_settings",programWindows.at(i)->curFile);
      settings.setValue("pos",programWindows.at(i)->pos());
      settings.setValue("size",programWindows.at(i)->size());
   }
   settings.endArray();

   setCurrentFile(fileName);
   statusBar()->clearMessage();
   return true;
}

bool kemboss::maybeSave()
{
   return true;
}

void kemboss::setCurrentFile(const QString &fileName)
{
   curFile = fileName;
   QSettings settings("kemboss.org", "kemboss");
   QStringList files = settings.value("recentFileList").toStringList();
   files.removeAll(fileName);
   files.prepend(fileName);
   while (files.size() > MaxRecentFiles)
      files.removeLast();
   settings.setValue("recentFileList", files);
   updateRecentFileActions();
   ksettings.setcurrentDir(QFileInfo(fileName).absolutePath());
   setWindowTitle(strippedName(curFile)+" - kemboss");
}

void kemboss::on_actionCloseAll_triggered()
{
   kembossWindow *currentProgramWindow;
   while (!programWindows.isEmpty()) {
     currentProgramWindow = programWindows.takeFirst();
     currentProgramWindow->close();
   }
}

void kemboss::on_RunProgramPushButton_clicked()
{
   QString newProgram = selectedProgram();
   if (!newProgram.isEmpty())
      createProgramWindow(newProgram);
}

kprogramWindow* kemboss::createProgramWindow(const QString &newProgram)
{
   kprogramWindow *newProgramWindow;
   if (newProgram == "prettyplot") {
       newProgramWindow = new kprettyplot(this);
   } else if (newProgram == "aligncopy") {
       newProgramWindow = new kaligncopy(this, 0);
   } else if (newProgram == "aligncopypair") {
       newProgramWindow = new kaligncopy(this, 1);
   } else if (newProgram == "antigenic") {
       newProgramWindow = new kantigenic(this);
   } else if (newProgram == "cirdna") {
       newProgramWindow = new kdrawmap(this, 1);
   } else if (newProgram == "consambig") {
       newProgramWindow = new kconsambig(this);
   } else if (newProgram == "cpgplot") {
       newProgramWindow = new kcpgplot(this, 0);
   } else if (newProgram == "cpgreport") {
       newProgramWindow = new kcpgreport(this, 0);
   } else if (newProgram == "dan") {
       newProgramWindow = new kdan(this);
   } else if (newProgram == "density") {
       newProgramWindow = new kdensity(this);
   } else if (newProgram == "digest") {
       newProgramWindow = new kdigest(this);
   } else if (newProgram == "distmat") {
       newProgramWindow = new kdistmat(this);
   } else if (newProgram == "dotmatcher") {
       newProgramWindow = new kdotplot(this, 0);
   } else if (newProgram == "dotpath") {
       newProgramWindow = new kdotplot(this, 1);
   } else if (newProgram == "dottub") {
       newProgramWindow = new kdotplot(this, 2);
   } else if (newProgram == "emma") {
       newProgramWindow = new kemma(this);
   } else if (newProgram == "eprimer3") {
       newProgramWindow = new keprimer3(this);
   } else if (newProgram == "est2genome") {
       newProgramWindow = new kest2genome(this);
   } else if (newProgram == "extractfeat") {
       newProgramWindow = new kextractfeat(this);
   } else if (newProgram == "fuzztran") {
       newProgramWindow = new kanalyzeseq(this, "fuzztran");
   } else if (newProgram == "featcopy") {
       newProgramWindow = new kfeatcopy(this);
   } else if (newProgram == "featreport") {
       newProgramWindow = new kfeatreport(this);
   } else if (newProgram == "getorf") {
       newProgramWindow = new kgetorf(this);
   } else if (newProgram == "helixturnhelix") {
       newProgramWindow = new khelixturnhelix(this);
   } else if (newProgram == "infobase") {
       newProgramWindow = new kinfo(this, 0);
   } else if (newProgram == "inforesidue") {
       newProgramWindow = new kinfo(this, 1);
   } else if (newProgram == "infoalign") {
       newProgramWindow = new kinfoalign(this);
   } else if (newProgram == "infoseq") {
       newProgramWindow = new kinfoseq(this);
   } else if (newProgram == "jaspscan") {
       newProgramWindow = new kjaspscan(this);
   } else if (newProgram == "lindna") {
       newProgramWindow = new kdrawmap(this, 0);
   } else if (newProgram == "maskambignuc") {
       newProgramWindow = new kmodifyseq(this, "maskambignuc");
   } else if (newProgram == "maskambigprot") {
       newProgramWindow = new kmodifyseq(this, "maskambigprot");
   } else if (newProgram == "maskfeat") {
       newProgramWindow = new kmaskfeat(this);
   } else if (newProgram == "matcher") {
       newProgramWindow = new kalignment(this, 3);
   } else if (newProgram == "merger") {
       newProgramWindow = new kmerger(this);
   } else if (newProgram == "msbar") {
       newProgramWindow = new kmsbar(this);
   } else if (newProgram == "needle") {
       newProgramWindow = new kalignment(this, 1);
   } else if (newProgram == "needleall") {
       newProgramWindow = new kalignment(this, 2);
   } else if (newProgram == "newcpgreport") {
       newProgramWindow = new kcpgplot(this, 1);
   } else if (newProgram == "newcpgseek") {
       newProgramWindow = new kcpgreport(this, 1);
   } else if (newProgram == "palindrome") {
       newProgramWindow = new kpalindrome(this);
   } else if (newProgram == "pepcoil") {
       newProgramWindow = new kpepcoil(this);
   } else if (newProgram == "polydot") {
       newProgramWindow = new kpolydot(this);
   } else if (newProgram == "plotorf") {
       newProgramWindow = new kplotorf(this);
   } else if (newProgram == "prettyseq") {
       newProgramWindow = new kprettyseq(this);
   } else if (newProgram == "primersearch") {
       newProgramWindow = new kprimersearch(this);
   } else if (newProgram == "prophecy") {
       newProgramWindow = new kprophecy(this);
   } else if (newProgram == "remap") {
       newProgramWindow = new kremap(this);
   } else if (newProgram == "restover") {
       newProgramWindow = new krestover(this);
   } else if (newProgram == "restrict") {
       newProgramWindow = new krestrict(this);
   } else if (newProgram == "showalign") {
       newProgramWindow = new kshowalign(this);
   } else if (newProgram == "showfeat") {
       newProgramWindow = new kshowfeat(this);
   } else if (newProgram == "showseq") {
       newProgramWindow = new kshowseq(this);
   } else if (newProgram == "showpep") {
       newProgramWindow = new kshowpep(this);
   } else if (newProgram == "sigcleave") {
       newProgramWindow = new ksigcleave(this);
   } else if (newProgram == "sirna") {
       newProgramWindow = new ksirna(this);
   } else if (newProgram == "sixpack") {
       newProgramWindow = new ksixpack(this);
   } else if (newProgram == "sizeseq") {
       newProgramWindow = new kmodifyseq(this, "sizeseq");
   } else if (newProgram == "skipredundant") {
       newProgramWindow = new kskipredundant(this);
   } else if (newProgram == "stretcher") {
       newProgramWindow = new kalignment(this, 5);
   } else if (newProgram == "supermatcher") {
       newProgramWindow = new kalignment(this, 4);
   } else if (newProgram == "tcode") {
       newProgramWindow = new ktcode(this);
   } else if (newProgram == "tranalign") {
       newProgramWindow = new ktranalign(this);
   } else if (newProgram == "transeq") {
       newProgramWindow = new ktranseq(this);
   } else if (newProgram == "twofeat") {
       newProgramWindow = new ktwofeat(this);
   } else if (newProgram == "union") {
       newProgramWindow = new kunion(this);
   } else if (newProgram == "water") {
       newProgramWindow = new kalignment(this, 0);
   } else if (newProgram == "backtranseq") {
       newProgramWindow = new kbacktranseq(this);
   } else if (newProgram == "backtranambig") {
       newProgramWindow = new kbacktranambig(this);
   } else if (newProgram == "charge") {
       newProgramWindow = new kcharge(this);
   } else if (newProgram == "checktrans") {
       newProgramWindow = new kchecktrans(this);
   } else if (newProgram == "fuzznuc") {
       newProgramWindow = new kanalyzeseq(this, "fuzznuc");
   } else if (newProgram == "hmoment") {
       newProgramWindow = new khmoment(this);
   } else if (newProgram == "maskseq") {
       newProgramWindow = new kmodifyseq(this, "maskseq");
   } else if (newProgram == "pepwheel") {
       newProgramWindow = new kpepdisplay(this, 0);
   } else if (newProgram == "pepinfo") {
       newProgramWindow = new kpepinfo(this);
   } else if (newProgram == "pepnet") {
       newProgramWindow = new kpepdisplay(this, 1);
   } else if (newProgram == "pepwindow") {
       newProgramWindow = new kpepwindow(this);
   } else if (newProgram == "pepwindowall") {
       newProgramWindow = new kpepwindow(this);
   } else if (newProgram == "plotcon") {
       newProgramWindow = new kplotcon(this);
   } else if (newProgram == "seqret") {
       newProgramWindow = new kseq(this, 0);
   } else if (newProgram == "skipseq") {
       newProgramWindow = new kseq(this, 2);
   } else if (newProgram == "seqretsplit") {
       newProgramWindow = new kseq(this, 1);
   } else if (newProgram == "revseq") {
       newProgramWindow = new kmodifyseq(this, "revseq");
   } else if (newProgram == "nthseq") {
       newProgramWindow = new kseq(this, 3);
   } else if (newProgram == "entret") {
       newProgramWindow = new kseq(this, 4);
   } else if (newProgram == "vectorstrip") {
       newProgramWindow = new kvectorstrip(this);
   } else if (newProgram == "sirna") {
       newProgramWindow = new ksirna(this);
   } else if (newProgram == "cons") {
       newProgramWindow = new kcons(this);
   } else if (newProgram == "abiview") {
       newProgramWindow = new kabiview(this);
   } else if (newProgram == "compseq") {
       newProgramWindow = new kcompseq(this);
   } else if (newProgram == "cutseq") {
       newProgramWindow = new kmodifyseq(this, "cutseq");
   } else if (newProgram == "degapseq") {
       newProgramWindow = new kmodifyseq(this, "degapseq");
   } else if (newProgram == "descseq") {
       newProgramWindow = new kmodifyseq(this, "descseq");
   } else if (newProgram == "diffseq") {
       newProgramWindow = new kdiffseq(this);
   } else if (newProgram == "dreg") {
       newProgramWindow = new kanalyzeseq(this, "dreg");
   } else if (newProgram == "emowse") {
       newProgramWindow = new kemowse(this);
   } else if (newProgram == "equicktandem") {
       newProgramWindow = new kequicktandem(this);
   } else if (newProgram == "etandem") {
       newProgramWindow = new ketandem(this);
   } else if (newProgram == "extractseq") {
       newProgramWindow = new kmodifyseq(this, "extractseq");
   } else if (newProgram == "freak") {
       newProgramWindow = new kfreak(this);
   } else if (newProgram == "fuzzpro") {
       newProgramWindow = new kanalyzeseq(this, "fuzzpro");
   } else if (newProgram == "iep") {
       newProgramWindow = new kiep(this);
   } else if (newProgram == "notseq") {
       newProgramWindow = new kmodifyseq(this, "notseq");
   } else if (newProgram == "megamerger") {
       newProgramWindow = new kmegamerger(this);
   } else if (newProgram == "pasteseq") {
       newProgramWindow = new kmodifyseq(this, "pasteseq");
   } else if (newProgram == "extractalign") {
       newProgramWindow = new kmodifyseq(this, "extractalign");
   } else if (newProgram == "patmatdb") {
       newProgramWindow = new kanalyzeseq(this, "patmatdb");
   } else if (newProgram == "preg") {
       newProgramWindow = new kanalyzeseq(this, "preg");
   } else if (newProgram == "recoder") {
       newProgramWindow = new kremutate(this, 1);
   } else if (newProgram == "seqmatchall") {
       newProgramWindow = new kanalyzeseq(this, "seqmatchall");
   } else if (newProgram == "silent") {
       newProgramWindow = new kremutate(this, 0);
   } else if (newProgram == "shuffleseq") {
       newProgramWindow = new kmodifyseq(this, "shuffleseq");
   } else if (newProgram == "syco") {
       newProgramWindow = new ksyco(this);
   } else if (newProgram == "splitter") {
       newProgramWindow = new kmodifyseq(this, "splitter");
   } else if (newProgram == "tmap") {
       newProgramWindow = new ktmap(this);
   } else if (newProgram == "trimest") {
       newProgramWindow = new ktrimest(this);
   } else if (newProgram == "trimseq") {
       newProgramWindow = new ktrimseq(this);
   } else if (newProgram == "wordcount") {
       newProgramWindow = new kanalyzeseq(this, "wordcount");
   } else if (newProgram == "banana") {
       newProgramWindow = new kbanana(this);
   } else if (newProgram == "isochore") {
       newProgramWindow = new kisochore(this);
   } else if (newProgram == "patmatmotifs") {
       newProgramWindow = new kpatmatmotifs(this);
   } else if (newProgram == "showorf") {
       newProgramWindow = new kshoworf(this);
   } else if (newProgram == "textsearch") {
       newProgramWindow = new ktextsearch(this);
   } else if (newProgram == "octanol") {
       newProgramWindow = new koctanol(this);
   } else if (newProgram == "biosed") {
       newProgramWindow = new kmodifyseq(this, "biosed");
   } else if  (newProgram == "pepstats") {
       newProgramWindow = new kanalyzeseq(this, "pepstats");
   } else if (newProgram == "garnier") {
       newProgramWindow = new kanalyzeseq(this, "garnier");
   } else if (newProgram == "cusp") {
       newProgramWindow = new kanalyzeseq(this, "cusp");
   } else if (newProgram == "marscan") {
       newProgramWindow = new kanalyzeseq(this, "marscan");
   } else if (newProgram == "pscan") {
       newProgramWindow = new kanalyzeseq(this, "pscan");
   } else if (newProgram == "geecee") {
       newProgramWindow = new kanalyzeseq(this, "geecee");
   } else if (newProgram == "cai") {
       newProgramWindow = new kanalyzeseq(this, "cai");
   } else if (newProgram == "chips") {
       newProgramWindow = new kanalyzeseq(this, "chips");
   } else if (newProgram == "btwisted") {
       newProgramWindow = new kbtwisted(this);
   } else if (newProgram == "wordmatch") {
       newProgramWindow = new kwordmatch(this);
   } else if (newProgram == "einverted") {
       newProgramWindow = new keinverted(this);
   } else if (newProgram == "coderet") {
       newProgramWindow = new kcoderet(this);
   } else if (newProgram == "wobble") {
       newProgramWindow = new kwobble(this);
   } else if (newProgram == "listor") {
       newProgramWindow = new klistor(this);
   } else if (newProgram == "tfscan") {
       newProgramWindow = new ktfscan(this);
   } else if (newProgram == "chaos") {
       newProgramWindow = new kchaos(this);
   } else if (newProgram == "newseq") {
       newProgramWindow = new knewseq(this);
   } else if (newProgram == "oddcomp") {
       newProgramWindow = new koddcomp(this);
   } else if (newProgram == "epestfind") {
       newProgramWindow = new kepestfind(this);
   } else if (newProgram == "prophet") {
       newProgramWindow = new kprophet(this);
   } else if (newProgram == "redata") {
       newProgramWindow = new kredata(this);
   } else if (newProgram == "profit") {
       newProgramWindow = new kprofit(this);
   } else if (newProgram == "findkm") {
       newProgramWindow = new kfindkm(this);
   } else if (newProgram == "mwfilter") {
       newProgramWindow = new kmwfilter(this);
   } else if (newProgram == "mwcontam") {
       newProgramWindow = new kmwcontam(this);
   } else if (newProgram == "whichdb") {
       newProgramWindow = new kwhichdb(this);
   } else if (newProgram == "codcmp") {
       newProgramWindow = new kcodcmp(this);
   } else if (newProgram == "codcopy") {
       newProgramWindow = new kcodcopy(this);
   } else if (newProgram == "nohtml") {
       newProgramWindow = new kno(this, 0);
   } else if (newProgram == "noreturn") {
       newProgramWindow = new kno(this, 1);
   } else if (newProgram == "nospace") {
       newProgramWindow = new kno(this, 2);
   } else if (newProgram == "notab") {
       newProgramWindow = new kno(this, 3);
   } else if (newProgram == "trimspace") {
       newProgramWindow = new kno(this, 4);
   } else if (newProgram == "makenucseq") {
       newProgramWindow = new kmakeseq(this, 1);
   } else if (newProgram == "makeprotseq") {
       newProgramWindow = new kmakeseq(this, 0);
   } else if (newProgram == "psiphi") {
       newProgramWindow = new kpsiphi(this);
   } else if (newProgram == "edialign") {
       newProgramWindow = new kedialign(this);
   } else if (newProgram == "wordfinder") {
       newProgramWindow = new kwordfinder(this);
   } else if (newProgram == "stssearch") {
       newProgramWindow = new kstssearch(this);
   } else if (newProgram == "fdnadist") {
       newProgramWindow = new kfdnadist(this);
   } else if (newProgram == "fprotdist") {
       newProgramWindow = new kfprotdist(this);
   } else if (newProgram == "fgendist") {
       newProgramWindow = new kfgendist(this);
   } else if (newProgram == "fclique") {
       newProgramWindow = new kfclique(this);
   } else if (newProgram == "fconsense") {
       newProgramWindow = new kfconsense(this);
   } else if (newProgram == "fcontml") {
       newProgramWindow = new kfcontml(this);
   } else if (newProgram == "fcontrast") {
       newProgramWindow = new kfcontrast(this);
   } else if (newProgram == "fdnacomp") {
       newProgramWindow = new kfdnacomp(this);
   } else if (newProgram == "fdnainvar") {
       newProgramWindow = new kfdnainvar(this);
   } else if (newProgram == "fdnaml") {
       newProgramWindow = new kfdnaml(this, 0);
   } else if (newProgram == "fdnamlk") {
       newProgramWindow = new kfdnaml(this, 1);
   } else if (newProgram == "fproml") {
       newProgramWindow = new kfdnaml(this, 2);
   } else if (newProgram == "fpromlk") {
       newProgramWindow = new kfdnaml(this, 3);
   } else if (newProgram == "fdollop") {
       newProgramWindow = new kfdollop(this, 0);
   } else if (newProgram == "ffactor") {
       newProgramWindow = new kffactor(this);
   } else if (newProgram == "ffitch") {
       newProgramWindow = new kfmatrix(this, 0);
   } else if (newProgram == "fkitsch") {
       newProgramWindow = new kfmatrix(this, 1);
   } else if (newProgram == "fneighbor") {
       newProgramWindow = new kfmatrix(this, 2);
   } else if (newProgram == "fmix") {
       newProgramWindow = new kfdollop(this, 1);
   } else if (newProgram == "fpars") {
       newProgramWindow = new kfpars(this, 0);
   } else if (newProgram == "fdnapars") {
       newProgramWindow = new kfpars(this, 1);
   } else if (newProgram == "fprotpars") {
       newProgramWindow = new kfpars(this, 2);
   } else if (newProgram == "fdnapenny") {
       newProgramWindow = new kfpenny(this, 0);
   } else if (newProgram == "fdolpenny") {
       newProgramWindow = new kfpenny(this, 1);
   } else if (newProgram == "fpenny") {
       newProgramWindow = new kfpenny(this, 2);
   } else if (newProgram == "frestml") {
       newProgramWindow = new kfrestml(this);
   } else if (newProgram == "ffreqboot") {
       newProgramWindow = new kfbootstrap(this, 0);
   } else if (newProgram == "frestboot") {
       newProgramWindow = new kfbootstrap(this, 1);
   } else if (newProgram == "fseqboot") {
       newProgramWindow = new kfbootstrap(this, 2);
   } else if (newProgram == "fseqbootall") {
       newProgramWindow = new kfbootstrap(this, 3);
   } else if (newProgram == "fdiscboot") {
       newProgramWindow = new kfdiscboot(this);
   } else if (newProgram == "frestdist") {
       newProgramWindow = new kfrestdist(this);
   } else if (newProgram == "ftreedist") {
       newProgramWindow = new kftreedist(this, 0);
   } else if (newProgram == "ftreedistpair") {
       newProgramWindow = new kftreedist(this, 1);
   } else if (newProgram == "fdrawgram") {
       newProgramWindow = new kfdraw(this, 0);
   } else if (newProgram == "fdrawtree") {
       newProgramWindow = new kfdraw(this, 1);
   } else {
      QMessageBox::warning(this, "kemboss", tr("A graphical interface for program '")+newProgram+tr("' is not yet available."));
      return NULL;
   }

   if (newProgramWindow) {
      programWindows.append(newProgramWindow);
      newProgramWindow->show();
      return newProgramWindow;
   }
   return NULL;
}

void kemboss::on_SearchByKeywordPushButton_clicked()
{
   ProgramsTreeWidget_3->clear();
   QStringList wossnameParameters;
   wossnameParameters.append(SearchByKeywordLineEdit->text());
   wossnameParameters.append("-noembassy");
   wossnameParameters.append("-gui");
   QProcess runWossname;
   runWossname.start(ksettings.embossBin()+"wossname", wossnameParameters);
   if (!runWossname.waitForFinished()) return; // waits 30 sec. for program wossname to finish
   QString programOutput(runWossname.readAll());
   QStringList programsFound = programOutput.split("\n", QString::SkipEmptyParts);
   for (int i = 1; i < programsFound.size(); ++i) {
      if (!programsFound.at(i).trimmed().isEmpty()) ProgramsTreeWidget_3->addTopLevelItem(new QTreeWidgetItem(ProgramsTreeWidget_3, QStringList() << programsFound.at(i).section(" ",0,0) << programsFound.at(i).section(" ",1).trimmed()));
   }
   ProgramsTreeWidget_3->sortItems(0,Qt::AscendingOrder);
   ProgramsTabWidget->setCurrentIndex(2);
}

void kemboss::on_PreferencesPushButton_clicked()
{
   configDlg ConfigDlgForm(this, &ksettings);
   ConfigDlgForm.exec();
}

void kemboss::on_actionEmbossHelp_triggered()
{
   QString currentProgram = selectedProgram();
   if (!currentProgram.isEmpty())
      assistant(currentProgram+".html");
   else
      assistant("index.html");
}

void kemboss::closeEvent(QCloseEvent *event)
{
   on_actionCloseAll_triggered();
   writeSettings();
   ksettings.writeSettings();
   QSettings settings("kemboss.org", "kemboss");
   if (ProgramsTabWidget->currentIndex() != 2) settings.setValue("view", ProgramsTabWidget->currentIndex());
   settings.setValue("tree_widget1_width_col1", ProgramsTreeWidget->columnWidth(0));
   settings.setValue("tree_widget1_width_col2", ProgramsTreeWidget->columnWidth(1));
   settings.setValue("tree_widget2_width_col1", ProgramsTreeWidget_2->columnWidth(0));
   settings.setValue("tree_widget2_width_col2", ProgramsTreeWidget_2->columnWidth(1));
   settings.setValue("tree_widget3_width_col1", ProgramsTreeWidget_3->columnWidth(0));
   settings.setValue("tree_widget3_width_col2", ProgramsTreeWidget_3->columnWidth(1));
   event->accept();
}
