/***************************************************************************
 *   Copyright (C) 2007 by Tobias Schwarz   *
 *   mail@tobias-schwarz.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/


#ifndef SETTINGS_H
#define SETTINGS_H

#include <QtGui>

class kembossSettings
{

   private:
   QString ckembossData;
   QString ckembossDefaultSettings;
   QString cembossData;
   QString cembossBin;
   QString cembossHelp;
   QString ccurrentDir;
   bool copenLastSettings;
   bool cautoCloseProgressWindow;
   bool cautoShowOutputTab;
   bool cautoDeleteExistingOutputFile;
   QFont ctextEditFont;
   bool cwrapLines;

   public:
   kembossSettings();
   virtual ~kembossSettings() {};
   void readSettings();
   void writeSettings() const;
   bool checkSettings();

   // get members
   QString kembossData() const {return ckembossData;}
   QString kembossDefaultSettings() const {return ckembossDefaultSettings;}
   QString embossData() const {return cembossData;}
   QString embossBin() const {return cembossBin;}
   QString embossHelp() const {return cembossHelp;}
   //QString embossAcd() const {return cembossAcd;}
   QString currentDir() const {return ccurrentDir;}
   bool openLastSettings() const {return copenLastSettings;}
   bool autoCloseProgressWindow() const {return cautoCloseProgressWindow;}
   bool autoShowOutputTab() const {return cautoShowOutputTab;}
   bool autoDeleteExistingOutputFile() const {return cautoDeleteExistingOutputFile;}
   QFont textEditFont() const {return ctextEditFont;}
   bool wrapLines() const { return cwrapLines;}

   // set members
   void setkembossData(const QString &dir) {ckembossData = QDir::toNativeSeparators(dir); if (!ckembossData.isEmpty() && !ckembossData.endsWith(QDir::separator())) ckembossData += QDir::separator(); }
   void setkembossDefaultSettings(const QString &dir) {ckembossDefaultSettings = QDir::toNativeSeparators(dir); if (!ckembossDefaultSettings.isEmpty() && !ckembossDefaultSettings.endsWith(QDir::separator())) ckembossDefaultSettings += QDir::separator(); }
   void setembossData(const QString &dir) {cembossData = QDir::toNativeSeparators(dir); if (!cembossData.isEmpty() && !cembossData.endsWith(QDir::separator())) cembossData += QDir::separator(); }
   void setembossBin(const QString &dir) {cembossBin = QDir::toNativeSeparators(dir); if (!cembossBin.isEmpty() && !cembossBin.endsWith(QDir::separator())) cembossBin += QDir::separator(); }
   void setembossHelp(const QString &dir) {cembossHelp = QDir::toNativeSeparators(dir); if (!cembossHelp.isEmpty() && !cembossHelp.endsWith(QDir::separator())) cembossHelp += QDir::separator(); }
   //void setembossAcd(const QString &dir) {cembossAcd = QDir::toNativeSeparators(dir); if (!cembossAcd.isEmpty() && !cembossAcd.endsWith(QDir::separator())) cembossAcd += QDir::separator(); }
   void setcurrentDir(const QString &dir) {ccurrentDir = QDir::toNativeSeparators(dir);}
   void setopenLastSettings(const bool &checked) {copenLastSettings = checked;}
   void setautoCloseProgressWindow(const bool &checked) {cautoCloseProgressWindow = checked;}
   void setautoShowOutputTab(const bool &checked) {cautoShowOutputTab = checked;}
   void setautoDeleteExistingOutputFile(const bool &checked) {cautoDeleteExistingOutputFile = checked;}
   void settextEditFont(const QString &family, int pointSize = -1, int weight = -1, bool italic = false) {QFont font(family, pointSize, weight, italic); ctextEditFont=font;}
   void setwrapLines(const bool &checked) {cwrapLines = checked;}
};

#endif
